/*
 * Copyright (c) 2000-2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * PROPRIETARY/CONFIDENTIAL
 *
 * This software is the confidential and proprietary information of
 * SAMSUNG ELECTRONICS ("Confidential Information").
 * You shall not disclose such Confidential Information and shall
 * use it only in accordance with the terms of the license agreement
 * you entered into with SAMSUNG ELECTRONICS.
 * SAMSUNG make no representations or warranties about the suitability
 * of the software, either express or implied, including but not
 * limited to the implied warranties of merchantability, fitness for
 * a particular purpose, or non-infringement.
 * SAMSUNG shall not be liable for any damages suffered by licensee as
 * a result of using, modifying or distributing this software or its derivatives.
 */

#ifndef __TIZEN_APPFW_REMOTE_APP_CONTROL_H__
#define __TIZEN_APPFW_REMOTE_APP_CONTROL_H__

#include <tizen.h>
#include <sys/types.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file remote_app_control.h
 */

/**
 * @brief Remote App Control handle.
 * @since_tizen 2.3.2
 */
typedef struct remote_app_control_s* remote_app_control_h;


/**
 * @brief Enumeration for Remote App Control Error.
 * @since_tizen 2.3.2
 */
typedef enum
{
	REMOTE_APP_CONTROL_ERROR_NONE = TIZEN_ERROR_NONE, /**< Successful */
	REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER = TIZEN_ERROR_INVALID_PARAMETER, /**< Invalid parameter */
	REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY = TIZEN_ERROR_OUT_OF_MEMORY, /**< Out of memory */
	REMOTE_APP_CONTROL_ERROR_PERMISSION_DENIED = TIZEN_ERROR_PERMISSION_DENIED,	/**< Permission denied */
	REMOTE_APP_CONTROL_ERROR_TIMED_OUT = TIZEN_ERROR_TIMED_OUT, /**< Time out */
	REMOTE_APP_CONTROL_ERROR_ALREADY_REQUESTED = TIZEN_ERROR_ALREADY_IN_PROGRESS, /**< Already requested */
	REMOTE_APP_CONTROL_ERROR_MAX_EXCEEDED = TIZEN_ERROR_QUOTA_EXCEEDED, /**< The size of the message has exceeded the maximum limit, the recommended data size is under 32KB */
	REMOTE_APP_CONTROL_ERROR_NOT_CONNECTED = TIZEN_ERROR_ENDPOINT_NOT_CONNECTED, /**< The remote device is not connected */
	REMOTE_APP_CONTROL_ERROR_NOT_SUPPORTED = TIZEN_ERROR_NOT_SUPPORTED, /**< Not supported in the remote device */
} remote_app_control_error_e;


/**
 * @brief Enumeration for Remote App Control Result.
 * @since_tizen 2.3.2
 */
typedef enum
{
	REMOTE_APP_CONTROL_RESULT_SUCCEEDED = 0,	 	/**< Requested operation succeeded by the callee application */
	REMOTE_APP_CONTROL_RESULT_FAILED = -1, 			/**< Requested operation failed by the calee application */
	REMOTE_APP_CONTROL_RESULT_APP_NOT_FOUND = -2,   /**< The remote application was not found in the remote device  */
} remote_app_control_result_e;


/**
 * @brief Definition for the remote app control operation: Displays the data.
 * @details Input: URI is usually used to inform the path to show (Replay callback is not supported). \n
 *      Output: Nothing\n
 * @since_tizen 2.3.2
 */
#define REMOTE_APP_CONTROL_OPERATION_VIEW "http://samsung.com/appcontrol/operation/remote/view"


/**
 * @brief Called when the reply of the launch request is delivered.
 *
 * @since_tizen 2.3.2
 * @remarks The @a request and @a reply must not be deallocated by the application.
 *
 * @param[in] request The remote app control handle of the launch request that has been sent
 * @param[in] reply The remote app control handle in which the results of the callee are contained
 * @param[in] result The result code of the launch request
 * @param[in] user_data	The user data passed from the callback registration function
 * @pre When the target application is launched or the callee replies to the launch request, this callback will be invoked.
 * @see remote_app_control_send_launch_request()
 */
typedef void (*remote_app_control_reply_cb) (remote_app_control_h request, remote_app_control_h reply, remote_app_control_result_e result, void *user_data);


/**
 * @addtogroup CAPI_REMOTE_APP_CONTROL_MODULE
 * @{
 */

/**
 * @brief Creates an remote app control handle.
 *
 * @since_tizen 2.3.2
 * @remarks The @a remote must be released using remote_app_control_destroy().
 * @param[out] remote The remote app control handle to be newly created on success
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
 * @see remote_app_control_destroy()
 */
int remote_app_control_create(remote_app_control_h *remote);

/**
 * @brief Destroys the remote app control handle and releases all its resources.
 *
 * @since_tizen 2.3.2
 * @param[in] remote The remote app control handle
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @see remote_app_control_create()
 */
int remote_app_control_destroy(remote_app_control_h remote);

/**
 * @brief Sets the operation to be performed.
 *
 * @since_tizen 2.3.2
 * @param[in] remote The remote app control handle
 * @param[in] operation The operation to be performed (if the @a operation is @c NULL, it clears the previous value)
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @see remote_app_control_get_operation()
 * @see REMOTE_APP_CONTROL_OPERATION_VIEW
 */
int remote_app_control_set_operation(remote_app_control_h app_control, const char *operation);


/**
 * @brief Gets the operation to be performed.
 *
 * @since_tizen 2.3.2
 * @remarks The @a operation must be released using free().
 * @param[in] remote The remote app control handle
 * @param[out] operation The operation to be performed
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
 * @see remote_app_control_set_operation()
 */
int remote_app_control_get_operation(remote_app_control_h app_control, char **operation);


/**
 * @brief Sets the Application name of the remote device to explicitly launch. (ex. setClassName() in Android Platform)
 *
 * @since_tizen 2.3.2
 * @param[in] remote The remote app control handle
 * @param[in] application The Application name of the remote device to explicitly launch (if the @a application is @c NULL, it clears  the previous value)
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 */
int remote_app_control_set_application(remote_app_control_h remote, const char *application);

/**
 * @brief Gets the Application name of the remote device to explicitly launch.
 *
 * @since_tizen 2.3.2
 * @remarks The @a application must be released with free().
 * @param[in] remotel The remote app control handle
 * @param[out] application The Application name to explicitly launch
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
 */
int remote_app_control_get_application(remote_app_control_h remote, char **application);

/**
 * @brief Sets the Package name of the remote device to explicitly launch. (ex. setPackage() in Android Platform)
 *
 * @since_tizen 2.3.2
 * @param[in] app_control The app_control handle
 * @param[in] package The Package name of the remote device to explicitly launch (if the @a package is @c NULL, it clears  the previous value)
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 */
int remote_app_control_set_package(remote_app_control_h app_control, const char *package);

/**
 * @brief Gets the Package name of the remote device to explicitly launch.
 *
 * @since_tizen 2.3.2
 * @remarks The @a application must be released with free().
 * @param[in] remote The remote app control handle
 * @param[out] package The Package name of the remote device to explicitly launch
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
 * @see remote_app_control_set_app_id()
 */
int remote_app_control_get_package(remote_app_control_h remote, char **package);

/**
 * @brief Sets the data to the remote app control.
 *
 * @since_tizen 2.3.2
 * @param[in] remote The remote app control handle
 * @param[in] data The data to be passed to the remote application, the recommended data size is under 32KB
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @see remote_app_control_get_data()
 */
int remote_app_control_set_data(remote_app_control_h remote, const char *data);

/**
 * @brief Gets the data from the app_control.
 *
 * @since_tizen 2.3.2
 * @remarks The @a value must be released using free().
 * @param[in] remote The remote app control handle
 * @param[out] data The data
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
*/
int remote_app_control_get_data(remote_app_control_h remote, char **data);

/**
 * @brief Sets the URI of the remote app control.
 *
 * @since_tizen 2.3.2
 * @param[in] remote The remote app control handle
 * @param[in] uri The URI of this remote app control is operating on (if the @a uri is @c NULL, it clears the previous value)
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @see remote_app_control_get_uri()
 */
int remote_app_control_set_uri(remote_app_control_h remote, const char *uri);

/**
 * @brief Gets the URI of the remote app control
 *
 * @since_tizen 2.3.2
 * @remarks The @a uri must be released using free().
 * @param[in] remote The remote app control handle
 * @param[out] uri The URI of this remote app control is operating on
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
 * @see remote_app_control_set_uri()
 */
int remote_app_control_get_uri(remote_app_control_h remote, char **uri);

/**
 * @brief Sends the launch request to the remote device.
 *
 * @since_tizen 2.3.2
 * @privlevel	public
 * @privilege   %http://tizen.org/privilege/appmanager.launch \n
 * 				%http://developer.samsung.com/tizen/privilege/accessoryprotocol
 * @param[in] app_control The app_control handle
 * @param[in] callback The callback function to be called when the reply is delivered
 * @param[in] user_data The user data to be passed to the callback function
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #REMOTE_APP_CONTROL_ERROR_NONE Successful
 * @retval #REMOTE_APP_CONTROL_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #REMOTE_APP_CONTROL_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #REMOTE_APP_CONTROL_ERROR_TIMED_OUT Failed due to timeout. The application that handles @a remote_app_control may be busy
 * @retval #REMOTE_APP_CONTROL_ERROR_PERMISSION_DENIED Permission denied
 * @retval #REMOTE_APP_CONTROL_ERROR_ALREADY_REQUESTED Already requested
 * @retval #REMOTE_APP_CONTROL_ERROR_MAX_EXCEEDED The size of data has exceeded the maximum limit, the recommended data size is under 32KB
 * @retval #REMOTE_APP_CONTROL_ERROR_NOT_CONNECTED The device is not connected
 * @retval #REMOTE_APP_CONTROL_ERROR_NOT_SUPPORTED Not supported in the remote device
 * @post If the launch request is sent for the result, the result will come back through remote_app_control_reply_cb() from the callee application.
 */
int remote_app_control_send_launch_request(remote_app_control_h remote, remote_app_control_reply_cb callback, void* user_data);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __TIZEN_APPFW_REMOTE_APP_CONTROL_H__ */
